#include "star.h"

#include "utils.h"

#include <alleggl.h>
#include <cmath>

static const int NumSlices = 12;
static const double StarRotateSpeed = 0.5; // Degrees per tick.
static const double PulseRate = 1.0 / 10.0; // Pulses per tick.
static const float PulseMagnitude = 0.2;

Star::Star()
{
	radius = 0.7;
	angle = FloatRand() * 360.0;
	pulseTimer = 0;
	
}

void Star::renderBackground()
{

}

void Star::render()
{
	glDisable(GL_BLEND);
	glDisable(GL_LIGHTING);
	glEnable(GL_TEXTURE_2D);
	glColor3f(1.0, 1.0,1.0);
	// bind our texture.
	glBindTexture(GL_TEXTURE_2D, 
		resourceManager.getTexture(ResourceManager::StarSurfaceTexture));
	glPushMatrix();
		glTranslatef(location.x, location.y, location.z);
		glRotatef(angle, 0,1,0);
		DrawSphere(Vect3d(0,0,0) , radius , NumSlices , 1 /* 0=quads, 1=tri */ ,
			0 /*theta1 */, 2 * M_PI /*theta2 */,
			- (M_PI / 2) /* phi1 */ , (M_PI / 2) /* phi2 */);
	glPopMatrix();
}

// Draw a glow on top.
// Assumption: depth testing disabled.
void Star::renderForeground()
{
	glEnable(GL_BLEND);
	glDisable(GL_LIGHTING);
	glEnable(GL_TEXTURE_2D);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	// Brightness
	float brightness = 0.5;
	if (pulseTimer > 0.5) {
		brightness = brightness + ((1.0 - pulseTimer) * PulseMagnitude * 2);
	} else {
		brightness = brightness + (pulseTimer * PulseMagnitude * 2);
	}
	glColor4f(1.0, 1.0, 1.0, brightness);

	glBindTexture(GL_TEXTURE_2D, 
		resourceManager.getTexture(ResourceManager::GlowTexture));
	
	glPushMatrix();
		float glowRadius = radius * 3;
		glTranslatef(location.x, location.y, location.z);
		glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex2f(-glowRadius,-glowRadius);
		glTexCoord2f(1,0);
		glVertex2f(glowRadius,-glowRadius);
		glTexCoord2f(1,1);
		glVertex2f(glowRadius,glowRadius);
		glTexCoord2f(0,1);
		glVertex2f(-glowRadius,glowRadius);
		glEnd();
	glPopMatrix();
	manager->showRadar(location, 1.0,1.0, 0.5); // yellowish
}

void Star::tick()
{
	// Nothing yet.
	angle += StarRotateSpeed;
	normaliseAngle(angle);
	
	pulseTimer += PulseRate;
	if (pulseTimer > 1.0) {
		pulseTimer -= 1.0;
	}
	
	location = location + velocity;
	manager->registerStarPos(this);
	ApplyFriction(velocity, 0.005);
}

void Star::collided(GameObject * what)
{
	// Create a big explosion.
	dead = true;
	manager->createExplosion(this);
}

